/* =====================================================================================
 * Header for ACTOframework
 * =====================================================================================
 * constants to be used across this project are defined here
 * see also ACTOControl or ACTO_framework for protocol
 *
 * date: 05. Mai 2014
 * version: v1.3
 *
 * project: ACTO
 * author: Emanuel Vonach
 * Interactive Media Systems Group, Vienna University of Technology, Austria
 * www.ims.tuwien.ac.at
 * Contact: vonach@ims.tuwien.ac.at, gerstweiler@ims.tuwien.ac.at
 * =====================================================================================
 */
#ifndef ACTOControl_h
#define ACTOControl_h

#include "Arduino.h"

///////////////////////////////////////////////////////////////////////////////////////
// ACTO SETTINGS to be concurrent across ACTO_framework (DO NOT TOUCH!)
///////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////
//  MESSAGES FOR SERIAL COMMUNICATION with ACTO_framework
//  single byte which specifies the command and thereby the package content
//
const byte USB_SEP = -111;		//use to separate messages

const byte USB_ACK = 40;		// ask/sending acknowledge			// format: CMD, SEP
const byte USB_NAK = 41; 		// negative acknowledge				// format: CMD, SEP

const byte USB_PING = 45; 		// are you online?				// format: CMD, ID, SEP
const byte USB_SYNC = 46;		// give me full sync of TUIO		        // format: CMD, ID, SEP

const byte USB_MOTOR = 50;		// motor unit on extension			// format: CMD, ID, direction(byte), speed(byte), SEP
const byte USB_MOTOR_MS = 51;		// motor unit on extension			//format: CMD, ID, direction(byte), speed(byte), time_ms(int), SEP

const byte USB_GET_DI_UNIT = 60;	// get state of digital unit on ext		//format: CMD, ID, dataID(byte), SEP
const byte USB_SET_DI_UNIT = 80;	// set state of digital unit on ext		//format: CMD, ID, dataID(byte), data(byte), SEP
const byte USB_SET_PWM_UNIT = 82;       // set state of digital unit on ext		//format: CMD, ID, dataID(byte), data(int), SEP

const byte USB_GET_AN_UNIT = 61;	// get state of analog unit on ext		//format: CMD, ID, dataID(byte), SEP
const byte USB_SET_AN_UNIT = 81;	// answer state of analog unit on ext		//format: CMD, ID, dataID(byte), data(int), SEP

const byte USB_GET_BYTE = 66;		// get byte data				//format: CMD, ID, dataID(byte), SEP
const byte USB_GET_CHAR = 67;		// get char data				//format: CMD, ID, dataID(byte), SEP
const byte USB_GET_INT = 68;		// get int data					//format: CMD, ID, dataID(byte), SEP
const byte USB_GET_FLOAT = 69;	        // get float data				//format: CMD, ID, dataID(byte), SEP

const byte USB_SET_BYTE = 86;		// send byte data				//format: CMD, ID, dataID(byte), data(byte), SEP
const byte USB_SET_CHAR = 87;		// send char data				//format: CMD, ID, dataID(byte), data(char16), SEP
const byte USB_SET_INT = 88;		// send integer data			        //format: CMD, ID, dataID(byte), data(int32), SEP
const byte USB_SET_FLOAT = 89;	        // send float data				//format: CMD, ID, dataID(byte), data(float32), SEP
//
///////////////////////////////////////////////////////////////////////////////////////
// ERROR definition, error Codes = number of blinks
//
#ifndef STATUSLED                        //check if STATUSLED is available
const int STATUSLED = -1;
#endif

const int SERIALERROR = 2;
const int MIRFERROR = 4;
const int SETUPERROR = 4;
const int MOTORERROR = 4;
const int DEBUG = 6;
const int BATTERYERROR = 10;

const unsigned long ERRORINTERVAL = 200;  //speed of blinking

// variables for battery checker
const int BATTERYPIN = A7;        //pin where checker is connected
const float BATTERYTHRESH = 7.45;  //stop working at this voltage
const unsigned long BATTERYINTERVAL = 1000;  //check battery at this ms

const float R2 = 4700.0;  //val of resistor of voltage divider
const float R1 = 4700.0;  //val of resistor of voltage divider
        
const int REFERENCEVOLTAGE = 5;    //working voltage of arduino
const float RESISTANCEFACTOR = 1023.0 / (R2/(R1+R2));  //factor for interpreting voltage on input
///////////////////////////////////////////////////////////////////////////////////////

//max payload of mirf messages in byte
const uint8_t mirfPayload = 9;


//ACTO variant of char
typedef union {
  //  char //1
  //  word //2
  wchar_t character;  //2
  byte binary[2];
} ActoChar;

//ACTO variant of int
typedef union {
  long integer;
  byte binary[4];
} ActoInt;

//ACTO variant of float
typedef union {
  float floatingPoint;
  byte binary[4];
} ActoFloat;

#endif
